"use strict";
var authorised_request_1 = require('./authorised-request');
var Rx_1 = require('rxjs/Rx');
var i_spy_1 = require('i-spy');
describe('SecureRequest', function () {
    describe('When the token is missing', function () {
        it('errors saying there is a token missing', function (done) {
            var subject = new Rx_1.Subject();
            subject.subscribe(fail, function (err) {
                expect(err.missingTokens).toEqual(true);
                done();
            });
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([null]); },
                events: subject
            };
            new authorised_request_1.SecureRequest(tokenStore, null)
                .authorisedRequest(null, function () { return Rx_1.Observable.from([]); })
                .subscribe(fail, fail);
        });
        it('it returns nothing if the token is missing', function (done) {
            var subject = new Rx_1.Subject();
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([null]); },
                events: subject
            };
            new authorised_request_1.SecureRequest(tokenStore, null)
                .authorisedRequest(null, function () { return Rx_1.Observable.from([]); })
                .subscribe(fail, fail, done);
        });
    });
    describe('When a token exists', function () {
        it('the callback is called', function (done) {
            var subject = new Rx_1.Subject();
            var token = {
                authToken: 'token',
                refreshToken: '',
                hasBeenRefreshed: false
            };
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([token]); },
                events: subject
            };
            var validator = {
                validateTokens: function (authToken, refreshToken) {
                    return Rx_1.Observable.from([token]);
                }
            };
            var whenAuthorised = i_spy_1.createSpy(function () { return Rx_1.Observable.from([{}]); });
            new authorised_request_1.SecureRequest(tokenStore, validator)
                .authorisedRequest({}, whenAuthorised)
                .subscribe(function () {
                expect(whenAuthorised.wasCalled()).toBe(true);
                done();
            });
        });
    });
    describe('When the token is invalid', function () {
        it('sends an api authorisation error', function (done) {
            var subject = new Rx_1.Subject();
            subject.subscribe(fail, function (err) {
                expect(err.invalidAuthToken).toEqual(true);
                done();
            });
            var token = {
                authToken: 'token',
                refreshToken: '',
                hasBeenRefreshed: false
            };
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([token]); },
                events: subject
            };
            var validator = {
                validateTokens: function (authToken, refreshToken) {
                    return Rx_1.Observable.throw(new Error('invalid token'));
                }
            };
            var whenAuthorised = i_spy_1.createSpy(function () { return Rx_1.Observable.from([{}]); });
            new authorised_request_1.SecureRequest(tokenStore, validator)
                .authorisedRequest({}, whenAuthorised)
                .subscribe(fail, fail);
        });
        it('returns an empty observable', function (done) {
            var subject = new Rx_1.Subject();
            var token = {
                authToken: 'token',
                refreshToken: '',
                hasBeenRefreshed: false
            };
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([token]); },
                events: subject
            };
            var validator = {
                validateTokens: function (authToken, refreshToken) {
                    return Rx_1.Observable.throw(new Error('invalid token'));
                }
            };
            var whenAuthorised = i_spy_1.createSpy(function () { return Rx_1.Observable.from([{}]); });
            new authorised_request_1.SecureRequest(tokenStore, validator)
                .authorisedRequest({}, whenAuthorised)
                .subscribe(fail, fail, done);
        });
    });
    describe('When the token is refreshed', function () {
        it('pushes a validated token event', function (done) {
            var whenAuthorised = i_spy_1.createSpy(function () { return Rx_1.Observable.from([{}]); });
            var subject = new Rx_1.Subject();
            var token = {
                authToken: 'token',
                refreshToken: '',
                hasBeenRefreshed: true
            };
            subject.subscribe(function (token) {
                expect(token).toEqual(token);
                done();
            }, fail);
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([token]); },
                events: subject
            };
            var validator = {
                validateTokens: function (authToken, refreshToken) {
                    return Rx_1.Observable.from([token]);
                }
            };
            new authorised_request_1.SecureRequest(tokenStore, validator)
                .authorisedRequest({}, whenAuthorised)
                .subscribe();
        });
        it('Calls the callback', function (done) {
            var whenAuthorised = i_spy_1.createSpy(function () { return Rx_1.Observable.from([{}]); });
            var subject = new Rx_1.Subject();
            var token = {
                authToken: 'token',
                refreshToken: '',
                hasBeenRefreshed: true
            };
            var tokenStore = {
                get: function () { return Rx_1.Observable.from([token]); },
                events: subject
            };
            var validator = {
                validateTokens: function (authToken, refreshToken) {
                    return Rx_1.Observable.from([token]);
                }
            };
            new authorised_request_1.SecureRequest(tokenStore, validator)
                .authorisedRequest({}, whenAuthorised)
                .subscribe(function () { }, function () { }, function () {
                expect(whenAuthorised.wasCalled()).toBe(true);
                done();
            });
        });
    });
});
//# sourceMappingURL=authorised-request.test.js.map